#version 130

#define PI 3.14159265358979323

varying vec3 ooPosition;
varying vec3 ooNormal;
varying vec3 ooViewDirection;
varying float ooRadius;

uniform vec3 inLight0Diffuse;
uniform vec3 inLight0Specular;
uniform vec3 inLight0Direction;
uniform sampler2D inBackgroundTexture;

uniform vec2 inBackgroundSize;
uniform vec3 inWaterColor = vec3(0.3, 0.3, 0.3);
const float MaterialSpecularPower = 80.0;

const float RefrIndexAir = 1.000293;
const float RefrIndexWater = 1.333;

const float BackgroundZ = -5.0;

const float Milkiness = 0.3;


float V3Angle(vec3 A, vec3 B) {
	return acos(dot(A, B) / length(A) / length(B));
}

vec3 V3Rotate(vec3 A, vec3 Axis, float Angle) {
	return Axis * dot(Axis, A) + cross(cross(Axis, A), Axis) * cos(Angle) + cross(Axis, A) * sin(Angle);
}

void main() {
	vec3 Normal = normalize(ooNormal);
	
	
	float alpha = V3Angle(-Normal, ooViewDirection);
	float beta = asin(sin(alpha) * RefrIndexAir / RefrIndexWater); // air -> water
	
	vec3 MegaNormal = normalize(cross(Normal, ooViewDirection));
	vec3 WaterDirection = V3Rotate(-Normal, MegaNormal, beta);
	float WaterDuration = 2.0 * ooRadius * cos(beta);
	
	vec3 WaterExit = ooPosition + WaterDirection * WaterDuration;
	vec3 WaterExitDirection = V3Rotate(+Normal, MegaNormal, -(2.0 * (PI - beta) + alpha));
	
	vec3 BackgroundHit = WaterExit + ((WaterExit.z - BackgroundZ) / WaterExitDirection.z) * WaterExitDirection;
	BackgroundHit.xy /= inBackgroundSize;
	BackgroundHit.y = 1.0 - BackgroundHit.y;
	vec3 BackgroundColor = vec3(texture2D(inBackgroundTexture, BackgroundHit.xy));
	
	vec3 SpecularNormal = vec3(0, 0, 1);
	vec3 DiffuseLight = vec3(0, 0, 0);
	vec3 SpecularLight = vec3(0, 0, 0);
	{
		float intensity = dot(Normal, inLight0Direction);
		if (intensity > 0.0) {
			DiffuseLight += inLight0Diffuse * intensity;
			vec3 HalfWay = normalize(ooViewDirection - normalize(inLight0Direction));
			float spectacular = dot(SpecularNormal, HalfWay);
			if (spectacular > 0.0)
				SpecularLight += inLight0Specular * pow(spectacular, MaterialSpecularPower);
		}
	}

	// sum up and put out
	gl_FragColor.rgb = BackgroundColor * (1.0 - Milkiness) + Milkiness * inWaterColor;
	gl_FragColor.rgb += inWaterColor * SpecularLight;
	gl_FragColor.a = 0.75;
}
