#version 130
#define maxNumOtherDrops 100

attribute vec3 inVertexPos;
attribute vec3 inVertexNormal;

varying vec3 ooPosition;
varying vec3 ooNormal;
varying vec3 ooViewDirection;
varying float ooRadius;

uniform mat4 inWorld;
uniform mat4 inView;
uniform mat4 inProj;
uniform vec3 inCamera;

uniform vec4 otherDrops[maxNumOtherDrops];
uniform int numOthers;
uniform int myIndex;
uniform vec3 velocity;

const float pullRange = 0.5;
const float force = 1;



vec3 getPullOffsetByIndex(int otherIndex, vec4 WorldPos){
	vec3 otherCenter = vec3(otherDrops[otherIndex]);
	vec3 myCenter = vec3(otherCenter[myIndex]);
	vec3 myVertex = vec3(WorldPos.x, WorldPos.y, 0.0);
	
	float distanceToVertex = distance(myVertex, otherCenter);
	float distanceToCenter = distance(myCenter, otherCenter);
	
	float modifier = 1.0 - (distanceToVertex/(pullRange + otherDrops[myIndex].w + otherDrops[otherIndex].w));
	
	if (modifier >= 0 && distanceToVertex < distanceToCenter){
		vec3 offset = otherCenter - myVertex;
		offset = normalize(offset);
		offset = offset * modifier * force * otherDrops[otherIndex].w;
		
		return offset;
	}else{
		return vec3(0,0,0);
	}
}

const float maxTailSpeed = 11.0;
const float maxTailLength = 1.5;

vec3 getTailOffset(vec4 WorldPos){
	vec3 myVertex = WorldPos.xyz;
	vec3 myCenter = otherDrops[myIndex].xyz;
	float myRadius = otherDrops[myIndex].w;
	
	float speed = length(velocity);
	vec3 velocityNormalized = normalize(velocity);
	vec3 pointInfront = myVertex + (velocityNormalized * 1000.0);
	
	float distanceToCenter = distance(pointInfront, myCenter);
	float distanceToVertex = distance(pointInfront, myVertex);
	
	if (distanceToVertex > distanceToCenter){
		float distanceVertexToCenter = distance(myVertex, myCenter);
		
		float speedMultiplierPow = 2.0;
		float speedMultiplier = pow(speed,speedMultiplierPow) / pow(maxTailSpeed,speedMultiplierPow);
		if (speedMultiplier > 1.0)
			speedMultiplier = 1.0;
		
		
		float tailLength = myRadius + myRadius * maxTailLength * speedMultiplier ;
		
		vec3 tailEndPoint = velocityNormalized * -1 * tailLength + myCenter;
		float howFarBetween = (distance(myVertex,myCenter) / myRadius) * speedMultiplier;
		
		
		vec3 pathVertexToTailEnd = tailEndPoint - myVertex;
		vec3 myOutput = pathVertexToTailEnd *howFarBetween;
		
		float newDistanceVertexToCenter = distance((myOutput+myVertex), myCenter);
		
		if (newDistanceVertexToCenter < distanceVertexToCenter)
			return vec3(0.0,0.0,0.0);
		else
			return myOutput;

	}else{
		return vec3(0.0,0.0,0.0);
	}
}

void main() {
	vec4 WorldPos = inWorld * vec4(inVertexPos,1.0);

	vec3 totalOffset = vec3(0.0,0.0,0.0);
	
	for (int i = 0; i < numOthers; i++){
		if (i != myIndex){
			totalOffset = totalOffset + getPullOffsetByIndex(i, WorldPos);
		}
	}
	
	
	
	
	vec4 newWorldPos = WorldPos + vec4(totalOffset,0.0) + vec4(getTailOffset(WorldPos),0.0);
	
	vec3 NewNormal = mat3(inWorld) * inVertexNormal;
	float OffsetLen = length(totalOffset);
	if (OffsetLen > 1.0)
		totalOffset /= length(totalOffset);
	NewNormal = normalize(NewNormal - totalOffset * 0.5);

	mat4 PVW = inProj * inView * inWorld;
	mat4 PV = inProj * inView;
	
	ooPosition = newWorldPos.xyz;
	ooNormal = mat3(PV) * NewNormal;
	ooViewDirection = normalize(inCamera - vec3(WorldPos));
	ooRadius = otherDrops[myIndex].w;

	gl_Position = PV * newWorldPos;
}
